@@ -240,8 +240,10 @@ typedef struct {
 */
 
 static void
-start_element (void *user_data, const xmlChar *name, const xmlChar **attrs)
-{
+start_element(void *           const user_data,
+              const xmlChar *  const name,
+              const xmlChar ** const attrs ATTR_UNUSED) {
+
     parse_context *context;
     xml_element *elem, *new_current;
 
@@ -281,9 +283,12 @@ start_element (void *user_data, const xmlChar *name, const xmlChar **attrs)
     }
 }
 
+
+
 static void
-end_element (void *user_data, const xmlChar *name)
-{
+end_element(void *          const user_data,
+            const xmlChar * const name ATTR_UNUSED) {
+
     parse_context *context;
 
     XMLRPC_ASSERT(user_data != NULL && name != NULL);
@@ -417,9 +422,11 @@ xml_parse(xmlrpc_env *   const envP,
     *resultPP = context.root;
 
  cleanup:
-    if (parser)
+    if (parser) {
+        if (parser->myDoc)
+            xmlFreeDoc(parser->myDoc);
         xmlFreeParserCtxt(parser);
-
+    }
     if (envP->fault_occurred) {
         if (context.root)
             xml_element_free(context.root);
